@@ -62,6 +62,14 @@ struct in_addr dest_ip;
 
 BOOL abort_mget = True;
 
+int Protocol = PROT_CORE;
+
+BOOL readbraw_supported = False;
+BOOL writebraw_supported = False;
+
+int tarbuffersize = 0;
+char *tarbuffer = NULL;
+
 /****************************************************************************
 change directory
 ****************************************************************************/
@@ -103,24 +111,24 @@ void cmd_cd(char *inbuf,char *outbuf )
 	  if ((errcls = CVAL(inbuf,smb_rcls) != 0))
 	    {
 	      errcode=SVAL(inbuf,smb_err);
-	      printf("cd %s : ", cur_dir);
+	      Debug(0,"cd %s : ", cur_dir);
 	      if(errcls == ERRDOS)
 		{
 		  if(errcode == eACCESS_DENIED)
-		    printf("permision denied\n");
+		    Debug(0,"permision denied\n");
 		  else if ( errcode == eINVALID_PATH)
-		    printf("no such directory\n");
+		    Debug(0,"no such directory\n");
 		  else
-		    printf("Error %d\n", errcode);
+		    Debug(0,"Error %d\n", errcode);
 		}
 	      else
-		printf("Error %d\n",errcode);
+		Debug(0,"Error %d\n",errcode);
 	      strcpy(cur_dir,saved_dir);
 	    }
 	}
     }
   else
-    printf("Current directory is %s\n",cur_dir);
+    Debug(0,"Current directory is %s\n",cur_dir);
 }
 
 /****************************************************************************
@@ -209,9 +217,9 @@ void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
 	  if (attr & aRONLY) strcat(attrstr,"R");
 
 	  if (fn)
-	    fn(name,attr,size,Date);
+	    fn(name,attr,size,Time);
 	  else
-	    printf("%20.20s%7.7s%7d  %s",name,attrstr,size,asctime(localtime(&Time)));
+	    Debug(0,"%20.20s%7.7s%7d  %s",name,attrstr,size,asctime(localtime(&Time)));
 	  p += DIR_STRUCT_SIZE;
 	}
     }
@@ -250,7 +258,7 @@ start a tar file if it isn't already open
 ****************************************************************************/
 BOOL starttar(void)
 {
-  if (tarfile) return;
+  if (tarfile) return(True);
 
   if (strequal(tarfilename,"-"))
     tarfile = stdout;
@@ -259,11 +267,18 @@ BOOL starttar(void)
 
   if (!tarfile)
     {
-      printf("Couldn't open tarfile %s\n",tarfilename);
+      Debug(0,"Couldn't open tarfile %s\n",tarfilename);
       abort_mget = True;
       return False;
     }
 
+  if (tarbuffersize > 0)
+    {
+      tarbuffer = (char *)malloc(tarbuffersize);
+      if (tarbuffer)
+	setbuffer(tarfile,tarbuffer,tarbuffersize);
+    }  
+
   return True;
 }
 
@@ -271,7 +286,7 @@ BOOL starttar(void)
 /****************************************************************************
 write a tar header
 ****************************************************************************/
-BOOL maketarhead(char *name,int dosmode,int size,uint32 dos_date)
+BOOL maketarhead(char *name,int dosmode,int size,time_t mtime)
 {
   union tarhblock tarhead;      
   pstring unixname;
@@ -279,18 +294,24 @@ BOOL maketarhead(char *name,int dosmode,int size,uint32 dos_date)
   
   strcpy(unixname,name);
   unix_format(unixname);
+  /* trim off any ./ then add it back */
   trim_string(unixname,"./",NULL);
+
+  {
+    pstring tmpnam;
+    strcpy(tmpnam,unixname);
+    strcpy(unixname,"./");
+    strcat(unixname,tmpnam);
+  }
+
   if (strlen(unixname) >= (NAMSIZ-1))
     {
-      printf("Name too long for tar format!\n");
+      Debug(0,"Name too long for tar format!\n");
       unixname[NAMSIZ-2]=0;
     }
 
   memset(&tarhead,0,sizeof(tarhead));
-  if (dosmode & aDIR)
-    sprintf(tarhead.dbuf.name,"%s/",unixname);
-  else
-    strncpy(tarhead.dbuf.name,unixname,NAMSIZ);
+  strncpy(tarhead.dbuf.name,unixname,NAMSIZ);
   
   
   if ((dosmode & aRONLY) == 0)
@@ -308,17 +329,18 @@ BOOL maketarhead(char *name,int dosmode,int size,uint32 dos_date)
   
   result &= ((~myumask) | S_IFDIR);
   
-  sprintf(tarhead.dbuf.mode,"%06.6o ",result);
-  sprintf(tarhead.dbuf.uid,"%06.6o ",uid);
-  sprintf(tarhead.dbuf.gid,"%06.6o ",gid);
-  sprintf(tarhead.dbuf.size,"%011.11o ",size);
-  sprintf(tarhead.dbuf.mtime,"%011.11o ",make_unix_date(dos_date));
-  memset(&tarhead.dbuf.chksum,' ',8);
-  sprintf(tarhead.dbuf.chksum,"%06.6o ",byte_checksum(&tarhead,sizeof(tarhead)));
+  sprintf(tarhead.dbuf.mode,"%06o ",result);
+  sprintf(tarhead.dbuf.uid,"%06o ",uid);
+  sprintf(tarhead.dbuf.gid,"%06o ",gid);
+  sprintf(tarhead.dbuf.size,"%011o ",size);
+  sprintf(tarhead.dbuf.mtime,"%011o ",mtime);
+  memset(tarhead.dbuf.chksum,' ',8);
+  sprintf(tarhead.dbuf.chksum,"%06o ",
+	  byte_checksum((unsigned char *)&tarhead,sizeof(tarhead)));
   
   if (fwrite(&tarhead,sizeof(tarhead),1,tarfile) != 1)
     {
-      printf("Error writing to tarfile\n");
+      Debug(0,"Error writing to tarfile\n");
       abort_mget = True;
       return(False);
     }      
@@ -329,7 +351,7 @@ BOOL maketarhead(char *name,int dosmode,int size,uint32 dos_date)
 /****************************************************************************
 get a file from rname to lname
 ****************************************************************************/
-void do_get(char *rname,char *lname,BOOL isdir)
+void do_get(char *rname,char *lname)
 {
   int handle,fnum;
   uint32 size,nread=0;
@@ -337,7 +359,7 @@ void do_get(char *rname,char *lname,BOOL isdir)
   BOOL newhandle = False;
   unsigned char *inbuf,*outbuf;
   int dosmode;
-  uint32 dos_date;
+  time_t mtime;
 
   if (lowercase)
     strlower(lname);
@@ -345,12 +367,12 @@ void do_get(char *rname,char *lname,BOOL isdir)
   if (DOTAR && !starttar())
     return;
 
-  inbuf = (unsigned char *)malloc(max_xmit + 4);
-  outbuf = (unsigned char *)malloc(max_xmit + 4);
+  inbuf = (unsigned char *)malloc(BUFFER_SIZE);
+  outbuf = (unsigned char *)malloc(BUFFER_SIZE);
 
   if (!inbuf || !outbuf)
     {
-      printf("out of memory\n");
+      Debug(0,"out of memory\n");
       return;
     }
 
@@ -381,8 +403,7 @@ void do_get(char *rname,char *lname,BOOL isdir)
 	}
       if (handle < 0)
 	{
-	  perror(lname);
-	  printf("Error opening local file %s\n",lname);
+	  Debug(0,"Error opening local file %s\n",lname);
 	  free(inbuf);free(outbuf);
 	  return;
 	}
@@ -393,7 +414,7 @@ void do_get(char *rname,char *lname,BOOL isdir)
 
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d opening remote file %s\n",SVAL(inbuf,smb_err),rname);
+      Debug(0,"Error %d opening remote file %s\n",SVAL(inbuf,smb_err),rname);
       if(newhandle)
 	close(handle);
       free(inbuf);free(outbuf);
@@ -403,62 +424,101 @@ void do_get(char *rname,char *lname,BOOL isdir)
   fnum = SVAL(inbuf,smb_vwv0);
   size = SVAL(inbuf,smb_vwv4) + (SVAL(inbuf,smb_vwv5)<<16);
   dosmode = SVAL(inbuf,smb_vwv1);
-  memcpy(&dos_date,inbuf + smb_vwv2,sizeof(dos_date));
+  mtime = SVAL(inbuf,smb_vwv2) | (SVAL(inbuf,smb_vwv3) << 16);
 
   if (tarfile)
-    printf("adding file %s of size %d bytes to %s\n",rname,size,tarfilename);
+      Debug(2,"adding file %s of size %d bytes to %s\n",rname,size,tarfilename);
   else
-    printf("getting file %s of size %d bytes as %s\n",rname,size,lname);
+      Debug(2,"getting file %s of size %d bytes as %s\n",rname,size,lname);
 
-  if (tarfile && !maketarhead(rname,dosmode,size,dos_date))
+  if (tarfile && !maketarhead(rname,dosmode,size,mtime))
     nread = size;
 
 
   while (nread < size)
     {
-      memset(outbuf,0,smb_size);
-      set_message(outbuf,5,0);
-      CVAL(outbuf,smb_com) = SMBread;
-      SSVAL(outbuf,smb_tid,cnum);
-      SSVAL(outbuf,smb_pid,pid);
-      SSVAL(outbuf,smb_uid,uid);
-      SSVAL(outbuf,smb_mid,mid);
-      SSVAL(outbuf,smb_vwv0,fnum);
-      SSVAL(outbuf,smb_vwv1,MIN(1024*((max_xmit-1024)/1024),size - nread));
-      SSVAL(outbuf,smb_vwv2,nread & 0xFFFF);
-      SSVAL(outbuf,smb_vwv3,nread >> 16);
-      SSVAL(outbuf,smb_vwv4,size - nread);
+      unsigned char *dataptr;
+      int datalen;
 
-      send_smb(outbuf);
-      receive_smb(inbuf);
+      if (readbraw_supported)
+	{
+	  extern int Client;
+	  memset(outbuf,0,smb_size);
+	  set_message(outbuf,8,0);
+	  CVAL(outbuf,smb_com) = SMBreadbraw;
+	  SSVAL(outbuf,smb_tid,cnum);
+	  SSVAL(outbuf,smb_pid,pid);
+	  SSVAL(outbuf,smb_uid,uid);
+	  SSVAL(outbuf,smb_mid,mid);
+	  SSVAL(outbuf,smb_vwv0,fnum);
+	  SIVAL(outbuf,smb_vwv1,nread);
+	  SSVAL(outbuf,smb_vwv3,MIN(size-nread,BUFFER_SIZE-4));
+	  SSVAL(outbuf,smb_vwv4,0);
+	  SIVAL(outbuf,smb_vwv5,1000);
+	  send_smb(outbuf);
 
-      if (CVAL(inbuf,smb_rcls) != 0)
-	{
-	  printf("Error %d reading remote file\n",SVAL(inbuf,smb_err));
-	  break;
-	}
-      
-      if (tarfile)
-	{
-	  if (fwrite(smb_buf(inbuf)+3,1,SVAL(inbuf,smb_vwv0),tarfile) != 
-	      SVAL(inbuf,smb_vwv0))
-	    {
-	      printf("Error writing to tarfile\n");
-	      abort_mget = True;
-	      break;
-	    }      
+	  /* Now read the raw data into the buffer and write it */	  
+	  if(!read_data(Client,inbuf,4)) {
+	    Debug(0,"Failed to read length in readbraw\n");	    
+	    exit(0);
+	  }
+
+	  /* Even though this is not an smb message, smb_len
+	     returns the generic length of an smb message */
+	  datalen = smb_len(inbuf);
+	  if(!read_data(Client,inbuf,datalen)) {
+	    Debug(0,"Failed to read data in readbraw\n");
+	    exit(1);
+	  }
+
+	  dataptr = inbuf;
 	  
 	}
       else
-	if (write(handle,smb_buf(inbuf)+3,SVAL(inbuf,smb_vwv0)) !=
-	    SVAL(inbuf,smb_vwv0))
+	{
+	  memset(outbuf,0,smb_size);
+	  set_message(outbuf,5,0);
+	  CVAL(outbuf,smb_com) = SMBread;
+	  SSVAL(outbuf,smb_tid,cnum);
+	  SSVAL(outbuf,smb_pid,pid);
+	  SSVAL(outbuf,smb_uid,uid);
+	  SSVAL(outbuf,smb_mid,mid);
+	  SSVAL(outbuf,smb_vwv0,fnum);
+	  SSVAL(outbuf,smb_vwv1,MIN(1024*((max_xmit-1024)/1024),size - nread));
+	  SSVAL(outbuf,smb_vwv2,nread & 0xFFFF);
+	  SSVAL(outbuf,smb_vwv3,nread >> 16);
+	  SSVAL(outbuf,smb_vwv4,size - nread);
+
+	  send_smb(outbuf);
+	  receive_smb(inbuf);
+
+	  if (CVAL(inbuf,smb_rcls) != 0)
+	    {
+	      Debug(0,"Error %d reading remote file\n",SVAL(inbuf,smb_err));
+	      break;
+	    }
+
+	  datalen = SVAL(inbuf,smb_vwv0);
+	  dataptr = smb_buf(inbuf) + 3;
+	}
+ 
+      if (tarfile)
+	{
+	  if (fwrite(dataptr,1,datalen,tarfile) != datalen)
+	    {
+	      Debug(0,"Error writing to tarfile\n");
+	      abort_mget = True;
+	      break;
+	    }      	  
+	}
+      else
+	if (write(handle,dataptr,datalen) != datalen)
 	  {
-	    perror(lname);
-	    printf("Error writing local file\n");
+	    Debug(0,"Error writing local file\n");
 	    break;
 	  }
       
-      nread += SVAL(inbuf,smb_vwv0);
+      nread += datalen;
     }
 
   if (tarfile)
@@ -470,7 +530,7 @@ void do_get(char *rname,char *lname,BOOL isdir)
 	  if (fwrite(outbuf,1,remainder,tarfile) != 
 	      remainder)
 	    {
-	      printf("Error writing to tarfile\n");
+	      Debug(0,"Error writing to tarfile\n");
 	      abort_mget = True;
 	    }      
 	}
@@ -492,7 +552,7 @@ void do_get(char *rname,char *lname,BOOL isdir)
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d closing remote file\n",SVAL(inbuf,smb_err));
+      Debug(0,"Error %d closing remote file\n",SVAL(inbuf,smb_err));
       if(newhandle)
 	close(handle);
       free(inbuf);free(outbuf);
@@ -520,7 +580,7 @@ void cmd_get(void)
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("get <filename>\n");
+      Debug(0,"get <filename>\n");
       return;
     }
   strcat(rname,p); 
@@ -531,14 +591,14 @@ void cmd_get(void)
   if (p)
     strcpy(lname,p);      
 
-  do_get(rname,lname,False);
+  do_get(rname,lname);
 }
 
 
 /****************************************************************************
 do a mget operation on one file
 ****************************************************************************/
-void do_mget(char *name,int attr,int size,uint32 dos_date)
+void do_mget(char *name,int attr,int size,time_t mtime)
 {
   pstring rname;
   pstring quest;
@@ -547,7 +607,10 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
     return;
 
   if (abort_mget)
-    return;
+    {
+      Debug(0,"mget aborted\n");
+      return;
+    }
 
   if (attr & aDIR)
     sprintf(quest,"Get directory %s? ",name);
@@ -562,12 +625,12 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
       pstring mget_mask;
       unsigned char *inbuf,*outbuf;
 
-      inbuf = (unsigned char *)malloc(max_xmit + 4);
-      outbuf = (unsigned char *)malloc(max_xmit + 4);
+      inbuf = (unsigned char *)malloc(BUFFER_SIZE);
+      outbuf = (unsigned char *)malloc(BUFFER_SIZE);
 
       if (!inbuf || !outbuf)
 	{
-	  printf("out of memory\n");
+	  Debug(0,"out of memory\n");
 	  return;
 	}
 
@@ -581,7 +644,7 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
 	{
 	  if (!directory_exist(name) && mkdir(name,0777) != 0) 
 	    {
-	      printf("failed to create directory %s\n",name);
+	      Debug(0,"failed to create directory %s\n",name);
 	      strcpy(cur_dir,saved_curdir);
 	      free(inbuf);free(outbuf);
 	      return;
@@ -589,7 +652,7 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
 
 	  if (chdir(name) != 0)
 	    {
-	      printf("failed to chdir to directory %s\n",name);
+	      Debug(0,"failed to chdir to directory %s\n",name);
 	      strcpy(cur_dir,saved_curdir);
 	      free(inbuf);free(outbuf);
 	      return;
@@ -597,7 +660,7 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
 	}       
       else
 	{
-	  if (!starttar() || !maketarhead(name,attr,0,dos_date))
+	  if (!starttar() || !maketarhead(cur_dir,attr,0,mtime))
 	    return;
 	}
 
@@ -616,7 +679,7 @@ void do_mget(char *name,int attr,int size,uint32 dos_date)
       strcpy(rname,cur_dir);
       strcat(rname,name);
 
-      do_get(rname,name,False);
+      do_get(rname,name);
     }
 }
 
@@ -658,13 +721,13 @@ BOOL do_mkdir(char *name)
   char *p;
   unsigned char *inbuf,*outbuf;
 
-  inbuf = (unsigned char *)malloc(max_xmit + 4);
-  outbuf = (unsigned char *)malloc(max_xmit + 4);
+  inbuf = (unsigned char *)malloc(BUFFER_SIZE);
+  outbuf = (unsigned char *)malloc(BUFFER_SIZE);
 
   if (!inbuf || !outbuf)
     {
-      printf("out of memory\n");
-      return;
+      Debug(0,"out of memory\n");
+      return False;
     }
 
   memset(outbuf,0,smb_size);
@@ -685,7 +748,7 @@ BOOL do_mkdir(char *name)
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d making remote directory file %s\n",SVAL(inbuf,smb_err),name);
+      Debug(0,"Error %d making remote directory file %s\n",SVAL(inbuf,smb_err),name);
       free(inbuf);free(outbuf);
       return(False);
     }
@@ -708,7 +771,7 @@ void cmd_mkdir(void)
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("mkdir <dirname>\n");
+      Debug(0,"mkdir <dirname>\n");
       return;
     }
   strcat(mask,p);
@@ -726,12 +789,12 @@ void do_put(char *rname,char *lname)
   char *p;
   unsigned char *inbuf,*outbuf;
 
-  inbuf = (unsigned char *)malloc(max_xmit + 4);
-  outbuf = (unsigned char *)malloc(max_xmit + 4);
+  inbuf = (unsigned char *)malloc(BUFFER_SIZE);
+  outbuf = (unsigned char *)malloc(BUFFER_SIZE);
 
   if (!inbuf || !outbuf)
     {
-      printf("out of memory\n");
+      Debug(0,"out of memory\n");
       return;
     }
 
@@ -757,7 +820,7 @@ void do_put(char *rname,char *lname)
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d opening remote file %s\n",SVAL(inbuf,smb_err),rname);
+      Debug(0,"Error %d opening remote file %s\n",SVAL(inbuf,smb_err),rname);
       free(inbuf);free(outbuf);
       return;
     }
@@ -765,8 +828,7 @@ void do_put(char *rname,char *lname)
   handle = open(lname,O_RDONLY);
   if (handle < 0)
     {
-      perror(lname);
-      printf("Error opening local file %s\n",lname);
+      Debug(0,"Error opening local file %s\n",lname);
       free(inbuf);free(outbuf);
       return;
     }
@@ -775,7 +837,7 @@ void do_put(char *rname,char *lname)
   fnum = SVAL(inbuf,smb_vwv0);
   size = file_size(lname);
   
-  printf("putting file %s of size %d bytes as %s\n",lname,size,rname);
+  Debug(0,"putting file %s of size %d bytes as %s\n",lname,size,rname);
   
   while (nread < size)
     {
@@ -798,8 +860,7 @@ void do_put(char *rname,char *lname)
 
       if (read(handle,smb_buf(outbuf)+3,n) != n)
 	{
-	  perror(lname);
-	  printf("Error reading local file\n");
+	  Debug(0,"Error reading local file\n");
 	  break;
 	}	  
 
@@ -808,14 +869,14 @@ void do_put(char *rname,char *lname)
 
       if (CVAL(inbuf,smb_rcls) != 0)
 	{
-	  printf("Error %d writing remote file\n",SVAL(inbuf,smb_err));
+	  Debug(0,"Error %d writing remote file\n",SVAL(inbuf,smb_err));
 	  break;
 	}
 
       
       if (n != SVAL(inbuf,smb_vwv0))
 	{
-	  printf("Error: only wrote %d bytes\n",nread + SVAL(inbuf,smb_vwv0));
+	  Debug(0,"Error: only wrote %d bytes\n",nread + SVAL(inbuf,smb_vwv0));
 	  break;
 	}
 
@@ -838,7 +899,7 @@ void do_put(char *rname,char *lname)
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d closing remote file\n",SVAL(inbuf,smb_err));
+      Debug(0,"Error %d closing remote file\n",SVAL(inbuf,smb_err));
       close(handle);
       free(inbuf);free(outbuf);
       return;
@@ -863,7 +924,7 @@ void cmd_put(void)
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("put <filename>\n");
+      Debug(0,"put <filename>\n");
       return;
     }
   strcpy(lname,p);
@@ -988,12 +1049,11 @@ void cmd_print(char *inbuf,char *outbuf )
   pstring lname;
   pstring rname;
   char *p;
-  BOOL first = True;
 
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("print <filename>\n");
+      Debug(0,"print <filename>\n");
       return;
     }
   strcpy(lname,p);
@@ -1038,7 +1098,7 @@ void cmd_print(char *inbuf,char *outbuf )
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d opening printer for %s\n",SVAL(inbuf,smb_err),rname);
+      Debug(0,"Error %d opening printer for %s\n",SVAL(inbuf,smb_err),rname);
       return;
     }
   
@@ -1046,15 +1106,14 @@ void cmd_print(char *inbuf,char *outbuf )
     f = fopen(lname,"r");
   if (!f)
     {
-      perror(lname);
-      printf("Error opening local file %s\n",lname);
+      Debug(0,"Error opening local file %s\n",lname);
       return;
     }
 
   
   fnum = SVAL(inbuf,smb_vwv0);
   
-  printf("printing file %s as %s\n",lname,rname);
+  Debug(1,"printing file %s as %s\n",lname,rname);
   
   while (!feof(f))
     {
@@ -1079,7 +1138,7 @@ void cmd_print(char *inbuf,char *outbuf )
 	  n = fread(smb_buf(outbuf)+3,1,n,f);
 	  if (n <= 0) 
 	    {
-	      printf("read gave %d\n",n);
+	      Debug(0,"read gave %d\n",n);
 	      break;
 	    }
 	}
@@ -1101,14 +1160,14 @@ void cmd_print(char *inbuf,char *outbuf )
 
       if (CVAL(inbuf,smb_rcls) != 0)
 	{
-	  printf("Error %d printing remote file\n",SVAL(inbuf,smb_err));
+	  Debug(0,"Error %d printing remote file\n",SVAL(inbuf,smb_err));
 	  break;
 	}
 
       nread += n;
     }
 
-  printf("%d bytes printed\n",nread);
+  Debug(2,"%d bytes printed\n",nread);
 
   memset(outbuf,0,smb_size);
   set_message(outbuf,1,0);
@@ -1124,7 +1183,7 @@ void cmd_print(char *inbuf,char *outbuf )
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d closing print file\n",SVAL(inbuf,smb_err));
+      Debug(0,"Error %d closing print file\n",SVAL(inbuf,smb_err));
       if (f != stdin)
 	fclose(f);
       return;
@@ -1149,7 +1208,7 @@ void cmd_del(char *inbuf,char *outbuf )
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("del <filename>\n");
+      Debug(0,"del <filename>\n");
       return;
     }
   strcat(mask,p);
@@ -1174,7 +1233,7 @@ void cmd_del(char *inbuf,char *outbuf )
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d deleting remote file %s\n",SVAL(inbuf,smb_err),mask);
+      Debug(0,"Error %d deleting remote file %s\n",SVAL(inbuf,smb_err),mask);
       return;
     }
   
@@ -1195,7 +1254,7 @@ void cmd_rmdir(char *inbuf,char *outbuf )
   p = strtok(NULL,SEPARATORS);
   if (!p)
     {
-      printf("rmdir <dirname>\n");
+      Debug(0,"rmdir <dirname>\n");
       return;
     }
   strcat(mask,p);
@@ -1219,7 +1278,7 @@ void cmd_rmdir(char *inbuf,char *outbuf )
   
   if (CVAL(inbuf,smb_rcls) != 0)
     {
-      printf("Error %d removing remote directory file %s\n",SVAL(inbuf,smb_err),mask);
+      Debug(0,"Error %d removing remote directory file %s\n",SVAL(inbuf,smb_err),mask);
       return;
     }
   
@@ -1231,7 +1290,7 @@ toggle the prompt flag
 void cmd_prompt(void)
 {
   prompt = !prompt;
-  printf("prompting is now %s\n",prompt?"on":"off");
+  Debug(2,"prompting is now %s\n",prompt?"on":"off");
 }
 
 /****************************************************************************
@@ -1240,7 +1299,7 @@ toggle the lowercaseflag
 void cmd_lowercase(void)
 {
   lowercase = !lowercase;
-  printf("filename lowercasing is now %s\n",lowercase?"on":"off");
+  Debug(2,"filename lowercasing is now %s\n",lowercase?"on":"off");
 }
 
 
@@ -1253,7 +1312,7 @@ void cmd_tar(void)
   if (p)
     strcpy(tarfilename,p);
 
-  printf("The tarfile is set to %s\n",tarfilename);
+  Debug(2,"The tarfile is set to %s\n",tarfilename);
 }
 
 
@@ -1267,9 +1326,15 @@ void cmd_endtar(void)
       char buf[TBLOCK*2];
       memset(buf,0,TBLOCK*2);
       if (fwrite(buf,TBLOCK,2,tarfile) != 2)
-	printf("Error writing to tarfile\n");
-      fclose(tarfile);
+	Debug(0,"Error writing to tarfile\n");
+      if (tarfile != stdout)
+	fclose(tarfile);
       tarfile = NULL;
+      if (tarbuffer)
+	{
+	  free(tarbuffer);
+	  tarbuffer = NULL;
+	}
     }
   strcpy(tarfilename,"");
 }
@@ -1280,7 +1345,7 @@ toggle the recurse flag
 void cmd_recurse(void)
 {
   recurse = !recurse;
-  printf("directory recursion is now %s\n",recurse?"on":"off");
+  Debug(2,"directory recursion is now %s\n",recurse?"on":"off");
 }
 
 
@@ -1290,7 +1355,6 @@ do a printmode command
 void cmd_printmode(void)
 {
   char *p;
-  pstring d;
   pstring mode;
 
   p = strtok(NULL,SEPARATORS);
@@ -1320,7 +1384,7 @@ void cmd_printmode(void)
       break;
     }
 
-  printf("the printmode is now %s\n",mode);
+  Debug(2,"the printmode is now %s\n",mode);
 }
 
 /****************************************************************************
@@ -1334,7 +1398,7 @@ void cmd_lcd(void)
   p = strtok(NULL,SEPARATORS);
   if (p)
     chdir(p);
-  printf("the local directory is now %s\n",GetWd(d));
+  Debug(2,"the local directory is now %s\n",GetWd(d));
 }
 
 
@@ -1343,11 +1407,21 @@ send a login command
 ****************************************************************************/
 BOOL send_login(char *inbuf,char *outbuf )
 {
-  char *prot = "PC NETWORK PROGRAM 1.0";
+  struct {
+    int prot;
+    char *name;
+  }
+  prots[] = 
+    {
+      {PROT_CORE,"PC NETWORK PROGRAM 1.0"},
+      {PROT_COREPLUS,"MICROSOFT NETWORKS 1.03"},
+      {-1,NULL}
+    };
   char *pass = NULL;  
   pstring dev = "A:";
   char *p;
   int len = 4;
+  int numprots;
 
   if (connect_as_printer)
     strcpy(dev,"LPT1:");
@@ -1383,9 +1457,26 @@ BOOL send_login(char *inbuf,char *outbuf )
       return(False);
     }      
 
-
   memset(outbuf,0,smb_size);
-  set_message(outbuf,0,2 + strlen(prot));
+
+  /* setup the protocol strings */
+  {
+    int plength;
+    char *p;
+
+    for (numprots=0,plength=0;prots[numprots].name;numprots++)
+      plength += strlen(prots[numprots].name)+2;
+    
+    set_message(outbuf,0,plength);
+
+    p = smb_buf(outbuf);
+    for (numprots=0;prots[numprots].name;numprots++)
+      {
+	*p++ = 2;
+	strcpy(p,prots[numprots].name);
+	p += strlen(p) + 1;
+      }
+  }
 
   CVAL(outbuf,smb_com) = SMBnegprot;
   SSVAL(outbuf,smb_pid,pid);
@@ -1393,12 +1484,10 @@ BOOL send_login(char *inbuf,char *outbuf )
   SSVAL(outbuf,smb_mid,mid);
   CVAL(smb_buf(outbuf),0) = 2;
 
-  strcpy(smb_buf(outbuf)+1,prot);
-
   send_smb(outbuf);
   receive_smb(inbuf);
 
-  if (CVAL(inbuf,smb_rcls) != 0)
+  if (CVAL(inbuf,smb_rcls) != 0 || (SVAL(inbuf,smb_vwv0) >= numprots))
     {
       Debug(0,"SMBnegprot failed error class 0x%x code 0x%x\n",
 	    CVAL(inbuf,smb_rcls),
@@ -1406,6 +1495,14 @@ BOOL send_login(char *inbuf,char *outbuf )
       return(False);
     }
 
+  Debug(3,"Chose protocol [%s]\n",prots[SVAL(inbuf,smb_vwv0)].name);
+  Protocol = prots[SVAL(inbuf,smb_vwv0)].prot;
+  if (Protocol >= PROT_COREPLUS)
+    {
+      readbraw_supported = ((SVAL(inbuf,smb_vwv5) & 0x1) != 0);
+      writebraw_supported = ((SVAL(inbuf,smb_vwv5) & 0x2) != 0);
+    }
+
   if (got_pass)
     pass = password;
   else
@@ -1447,7 +1544,7 @@ BOOL send_login(char *inbuf,char *outbuf )
 
   cnum = SVAL(inbuf,smb_vwv1);
 
-  Debug(1,"Connected with cnum=%d max_xmit=%d\n",cnum,max_xmit);
+  Debug(3,"Connected with cnum=%d max_xmit=%d\n",cnum,max_xmit);
 
   /* wipe out the password from memory */
   if (got_pass)
@@ -1538,14 +1635,14 @@ void cmd_help(void)
       while (commands[i].fn)
 	{
 	  if (strequal(commands[i].name,p))	  
-	    printf("HELP %s:\n\t%s\n\n",commands[i].name,commands[i].description);
+	    Debug(0,"HELP %s:\n\t%s\n\n",commands[i].name,commands[i].description);
 	  i++;
 	}
     }
   else
     while (commands[i].fn)
       {
-	printf("%s\n",commands[i].name);
+	Debug(0,"%s\n",commands[i].name);
 	i++;
       }
 }
@@ -1583,7 +1680,7 @@ BOOL open_sockets(int port )
   Client = socket(PF_INET, SOCK_STREAM, 0);
   if (Client == -1) 
     {
-      perror("socket");
+      Debug(0,"socket error\n");
       return False;
     }
   
@@ -1596,7 +1693,7 @@ BOOL open_sockets(int port )
   /* and connect it to the destination */
   if (connect(Client,(struct sockaddr *)&sock_out,sizeof(sock_out))<0)
     {
-      perror("connect");
+      Debug(0,"connect error\n");
       close(Client);
       return False;
     }
@@ -1610,8 +1707,6 @@ BOOL open_sockets(int port )
 ****************************************************************************/
 void process(void )
 {
-  static int trans_num = 0;
-  int nread;
   pstring line;
 
   InBuffer = (char *)malloc(BUFFER_SIZE);
@@ -1632,7 +1727,7 @@ void process(void )
       memset(OutBuffer,0,smb_size);
 
       /* display a prompt */
-      printf("smb: %s> ", cur_dir);
+      Debug(1,"smb: %s> ", cur_dir);
 
       /* and get a response */
       if (!fgets(line,1000,stdin))
@@ -1652,7 +1747,7 @@ void process(void )
 	  i++;
 	}
       if (!found && tok)
-	printf("%s: command not found\n",tok);
+	Debug(0,"%s: command not found\n",tok);
     }
   
   memset(OutBuffer,0,smb_size);
@@ -1665,15 +1760,17 @@ usage on the program
 ****************************************************************************/
 void usage(char *pname)
 {
-  printf("Usage: %s service <password> [-p port] [-d debuglevel] [-l log]\n",pname);
-  printf("\t-p port               listen on the specified port\n");
-  printf("\t-d debuglevel         set the debuglevel\n");
-  printf("\t-l log basename.      Basename for log/debug files\n");
-  printf("\t-n netbios name.      Use this name as my netbios name\n");
-  printf("\t-N                    don't ask for a password\n");
-  printf("\t-P                    connect to service as a printer\n");
-  printf("\t-I dest IP            use this IP to connect to\n");
-  printf("\n");
+  Debug(0,"Usage: %s service <password> [-p port] [-d debuglevel] [-l log]\n",pname);
+  Debug(0,"\t-p port               listen on the specified port\n");
+  Debug(0,"\t-d debuglevel         set the debuglevel\n");
+  Debug(0,"\t-l log basename.      Basename for log/debug files\n");
+  Debug(0,"\t-n netbios name.      Use this name as my netbios name\n");
+  Debug(0,"\t-N                    don't ask for a password\n");
+  Debug(0,"\t-P                    connect to service as a printer\n");
+  Debug(0,"\t-I dest IP            use this IP to connect to\n");
+  Debug(0,"\t-E                    write messages to stderr instead of stdout\n");
+  Debug(0,"\t-B tarbuffersize      set the buffer size for tar (in kbytes)\n");
+  Debug(0,"\n");
 }
 
 
@@ -1688,6 +1785,9 @@ int main(int argc,char *argv[])
   extern int DEBUGLEVEL;
   extern char *optarg;
 
+  DEBUGLEVEL = 2;
+  dbf = stdout;
+
   pid = getpid();
   uid = getuid();
   gid = getgid();
@@ -1712,9 +1812,15 @@ int main(int argc,char *argv[])
       argv++;
     }
 
-  while ((opt = getopt (argc, argv, "Nn:d:Pp:l:hI:")) != EOF)
+  while ((opt = getopt (argc, argv, "Nn:d:Pp:l:hI:EB:")) != EOF)
     switch (opt)
       {
+      case 'B':
+	tarbuffersize = atoi(optarg)*1024;
+	break;
+      case 'E':
+	dbf = stderr;
+	break;
       case 'I':
 	{
 	  unsigned long a = inet_addr(optarg);
@@ -1752,11 +1858,7 @@ int main(int argc,char *argv[])
   
   NeedSwap = big_endian();
   
-  if (DEBUGLEVEL > 0)
-    {
-      dbf = stdout;
-      Debug(1,"%s client started\n",timestring());
-    }
+  Debug(3,"%s client started\n",timestring());
 
   if (DEBUGLEVEL > 2)
     {
