@@ -309,6 +309,8 @@ service
 ***************************************************************************/
 static int add_a_service(service *pservice)
 {
+  service tservice = *pservice;
+
   if (iNumServices == 0)
     pServices = (service *)malloc(sizeof(service));
   else
@@ -326,7 +328,7 @@ static int add_a_service(service *pservice)
   if (pServices)
     {
       init_service(&pServices[iNumServices]);
-      copy_service(&pServices[iNumServices],pservice,NULL);
+      copy_service(&pServices[iNumServices],&tservice,NULL);
       iNumServices++;
     }
 
@@ -354,7 +356,7 @@ BOOL lp_add_home(char *pservice,int ifrom,char *phome)
   if (!pServices[inum].bGuest_ok)
     string_set(&pServices[inum].szUsername,pservice);
 
-  Debug(1,"adding home directory %s at %s\n",pservice,phome);
+  Debug(2,"adding home directory %s at %s\n",pservice,phome);
 
   return(True);
 }
@@ -420,7 +422,7 @@ static void map_parameter(char *pszParmName, parmmap *pparmmap)
          break;
 
    if (iIndex < 1)
-      Debug(1, "Unknown parameter encountered: \"%s\"\n", pszParmName);
+      Debug(0, "Unknown parameter encountered: \"%s\"\n", pszParmName);
 
    *pparmmap = aParmMaps[iIndex];
 }
@@ -716,7 +718,7 @@ static BOOL do_service_parameter(parmmap *pparmmap, char *pszParmValue)
          bRetval = False;
          string_set(&COPYMAPS(iServiceIndex).szSourceService, pszParmValue);
          COPYMAPS(iServiceIndex).bCopyEncountered = True;
-	 Debug(0,"Copying service from service %s\n",pszParmValue);
+	 Debug(2,"Copying service from service %s\n",pszParmValue);
          if ((iTemp = getservicebyname(pszParmValue, &serviceTemp)) >= 0)
          {
             if (iTemp == iServiceIndex)
