@@ -33,7 +33,8 @@ FILE *logout=NULL;
 int Client = 0;
 
 /* info on the client */
-struct from_host Client_info;
+struct from_host Client_info=
+{"UNKNOWN","0.0.0.0",NULL};
 
 /* the last IP received from */
 struct in_addr lastip;
@@ -160,6 +161,7 @@ while (n--)
 }
 #endif
 
+
 /****************************************************************************
   close the socket communication
 ****************************************************************************/
@@ -215,7 +217,7 @@ interpret the weird netbios "name"
 ****************************************************************************/
 void name_interpret(char *in,char *out)
 {
-char *p = in;
+
 int len = (*in++) / 2;
 while (len--)
   {
@@ -289,24 +291,24 @@ void object_byte_swap(void *obj,int size)
 }
 
 /****************************************************************************
-  byte swap a int16
+  byte swap a uint16
 ****************************************************************************/
-int16 int16_byte_swap(int16 x)
+uint16 uint16_byte_swap(uint16 x)
 {
-  int16 res;
+  uint16 res;
   res = x;
-  object_byte_swap(&res,sizeof(int16));
+  object_byte_swap(&res,sizeof(res));
   return(res);
 }
 
 /****************************************************************************
-  byte swap a int32
+  byte swap a uint32
 ****************************************************************************/
-int32 int32_byte_swap(int32 x)
+uint32 uint32_byte_swap(uint32 x)
 {
-  int32 res;
+  uint32 res;
   res = x;
-  object_byte_swap(&res,sizeof(int32));
+  object_byte_swap(&res,sizeof(res));
   return(res);
 }
 
@@ -493,7 +495,7 @@ void dos_format(char *fname)
 void sival(char *buf,int pos,uint32 val)
 {
   val = ISWP(val);
-  memcpy(buf + pos,(char *)&val,sizeof(int));
+  memcpy(buf + pos,(char *)&val,sizeof(val));
 }
 
 /****************************************************************************
@@ -508,9 +510,9 @@ void ssval(char *buf,int pos,uint16 val)
 /****************************************************************************
   get a 32 bit integer value
 ****************************************************************************/
-int ival(char *buf,int pos)
+uint32 ival(char *buf,int pos)
 {
-  int res;
+  uint32 res;
   memcpy((char *)&res,buf + pos,sizeof(int));
   res = ISWP(res);
   return(res);
@@ -535,22 +537,22 @@ uint16 sval(char *buf,int pos)
 void show_msg(char *buf)
 {
   int i;
-  Debug(1,"size=%d\nsmb_com=0x%x\nsmb_rcls=%d\nsmb_reh=%d\nsmb_err=%d\n",
+  Debug(3,"size=%d\nsmb_com=0x%x\nsmb_rcls=%d\nsmb_reh=%d\nsmb_err=%d\n",
 	  smb_len(buf),
 	  (int)CVAL(buf,smb_com),
 	  (int)CVAL(buf,smb_rcls),
 	  (int)CVAL(buf,smb_reh),
 	  (int)SVAL(buf,smb_err));
-  Debug(1,"smb_tid=%d\nsmb_pid=%d\nsmb_uid=%d\nsmb_mid=%d\nsmt_wct=%d\n",
+  Debug(3,"smb_tid=%d\nsmb_pid=%d\nsmb_uid=%d\nsmb_mid=%d\nsmt_wct=%d\n",
 	  (int)SVAL(buf,smb_tid),
 	  (int)SVAL(buf,smb_pid),
 	  (int)SVAL(buf,smb_uid),
 	  (int)SVAL(buf,smb_mid),
 	  (int)CVAL(buf,smb_wct));
   for (i=0;i<(int)CVAL(buf,smb_wct);i++)
-    Debug(1,"smb_vwv[%d]=%d\n",i,(int)SVAL(buf,smb_vwv+2*i));
-  Debug(1,"smb_bcc=%d\n",(int)SVAL(buf,smb_vwv+2*(CVAL(buf,smb_wct))));
-  
+    Debug(3,"smb_vwv[%d]=%d (0x%X)\n",i,
+	  SVAL(buf,smb_vwv+2*i),SVAL(buf,smb_vwv+2*i));
+  Debug(3,"smb_bcc=%d\n",(int)SVAL(buf,smb_vwv+2*(CVAL(buf,smb_wct))));
 }
 
 /*******************************************************************
@@ -626,7 +628,7 @@ void clean_name(char *s)
 {
   char *p=NULL;
 
-  Debug(2,"clean_name [%s]\n",s);
+  Debug(3,"clean_name [%s]\n",s);
 
   /* remove any double slashes */
   while ((p = strstr(s,"\\\\")) != NULL)
@@ -702,7 +704,7 @@ return True;
   char *p=NULL;
   BOOL relative = (*s != '/');
 
-  Debug(2,"reduce_name [%s] [%s]\n",s,dir);
+  Debug(3,"reduce_name [%s] [%s]\n",s,dir);
 
   /* remove any double slashes */
   while ((p = strstr(s,"//")) != NULL)
@@ -716,19 +718,19 @@ return True;
 
   if (!GetWd(wd))
     {
-      Debug(1,"couldn't getwd for %s %s\n",s,dir);
+      Debug(0,"couldn't getwd for %s %s\n",s,dir);
       return(False);
     }
 
   if (chdir(dir) != 0)
     {
-      Debug(3,"couldn't chdir to %s\n",dir);
+      Debug(0,"couldn't chdir to %s\n",dir);
       return(False);
     }
 
   if (!GetWd(dir2))
     {
-      Debug(1,"couldn't getwd for %s\n",dir);
+      Debug(0,"couldn't getwd for %s\n",dir);
       chdir(wd);
       return(False);
     }
@@ -753,7 +755,7 @@ return True;
   if (!GetWd(newname))
     {
       chdir(wd);
-      Debug(1,"couldn't get wd for %s %s\n",s,dir2);
+      Debug(2,"couldn't get wd for %s %s\n",s,dir2);
       return(False);
     }
 
@@ -790,7 +792,7 @@ return True;
   if (strlen(s) == 0)
     strcpy(s,"./");
 
-  Debug(2,"reduced to %s\n",s);
+  Debug(3,"reduced to %s\n",s);
   return(True);
 #endif
 }
@@ -808,7 +810,7 @@ BOOL mask_match(char *name,char *mask,BOOL dodots)
   pstring mext=""; /* extension of mask */
   pstring mbeg=""; /* beg of mask */  
     
-  Debug(2,"mmatch [%s] [%s] %d\n",name,mask,dodots);
+  Debug(3,"mmatch [%s] [%s] %d\n",name,mask,dodots);
 
   strcpy(mbeg,mask);
   if ((p1 = strchr(mbeg,'.')) != NULL)
@@ -1001,7 +1003,7 @@ int read_socket(int fd,char *buf,int len)
   else
     {
       lastip = *(struct in_addr *) &sock.sa_data[2];
-      Debug(2,"read %d bytes from (%s)\n",ret,inet_ntoa(lastip));
+      Debug(3,"read %d bytes from (%s)\n",ret,inet_ntoa(lastip));
     }
   return(ret);
 #endif
@@ -1481,7 +1483,7 @@ BOOL receive_nmb(char *buffer,int timeout)
 
   log_in(buffer,ret);
 
-  Debug(2,"nmb_len=%d len=%d\n",nmb_len(buffer),ret);
+  Debug(3,"nmb_len=%d len=%d\n",nmb_len(buffer),ret);
 
   return(True);
 }
@@ -1707,7 +1709,7 @@ BOOL send_packet(char *buf,int len,struct in_addr *ip,int port,int type)
   log_out(buf,len);
 
 
-  Debug(2,"sending a packet of len %d to (%s) on port %d of type %s\n",
+  Debug(3,"sending a packet of len %d to (%s) on port %d of type %s\n",
 	len,inet_ntoa(*ip),port,type==SOCK_DGRAM?"DGRAM":"STREAM");
 	
   /* send it */
@@ -1878,7 +1880,7 @@ BOOL get_broadcast(struct in_addr *if_ipaddr, struct in_addr *if_bcast)
   ifr = ifc.ifc_req;
   for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
     {
-      Debug(2,"Interface: %s  IP addr: %s\n", ifr->ifr_name,
+      Debug(3,"Interface: %s  IP addr: %s\n", ifr->ifr_name,
 	    inet_ntoa((*(struct sockaddr_in *) &ifr->ifr_addr).sin_addr));
       if (if_ipaddr->s_addr == (*(struct sockaddr_in *) &ifr->ifr_addr).sin_addr.s_addr)
         break;
@@ -1930,7 +1932,6 @@ do a netbios name query to find someones IP
 ****************************************************************************/
 BOOL name_query(char *inbuf,char *outbuf,char *name,struct in_addr *ip)
 {
-  unsigned char nb_flags = 0;
   int count;
   static uint16 name_trn_id = 0x6242;
   char *p;
@@ -1938,7 +1939,7 @@ BOOL name_query(char *inbuf,char *outbuf,char *name,struct in_addr *ip)
 
   NeedSwap = !big_endian();
 
-  Debug(1,"Querying name %s\n",name);
+  Debug(2,"Querying name %s\n",name);
 
   SSVAL(outbuf,0,name_trn_id++);
   CVAL(outbuf,2) = 0x1;
@@ -1957,7 +1958,7 @@ BOOL name_query(char *inbuf,char *outbuf,char *name,struct in_addr *ip)
   count = 3;
   while (count--)
     {
-      Debug(1,"Sending name query for %s\n",name);
+      Debug(2,"Sending name query for %s\n",name);
 
       show_nmb(outbuf);
       if (!send_packet(outbuf,nmb_len(outbuf),&bcast_ip,137,SOCK_DGRAM))
@@ -1972,11 +1973,6 @@ BOOL name_query(char *inbuf,char *outbuf,char *name,struct in_addr *ip)
 	  int opcode = (CVAL(inbuf,2) >> 3) & 0xF;
 	  int nm_flags = ((CVAL(inbuf,2) & 0x7) << 4) + (CVAL(inbuf,3)>>4);
 	  int rcode = CVAL(inbuf,3) & 0xF;
-	  int qdcount = SVAL(inbuf,4);
-	  int ancount = SVAL(inbuf,6);
-	  int nscount = SVAL(inbuf,8);
-	  int arcount = SVAL(inbuf,10);
-
 	  show_nmb(inbuf);
 
 	  /* is it a positive response to our request? */
@@ -2106,3 +2102,68 @@ int byte_checksum(unsigned char *buf,int len)
     ret += *buf++;
   return(ret);
 }
+
+
+/****************************************************************************
+check if a username/password is OK
+****************************************************************************/
+BOOL password_ok(char *user,char *password)
+{
+  BOOL pwok = False;
+  char salt[10];
+#ifdef SHADOW_PWD
+  struct spwd *spass = NULL;
+#endif
+  struct passwd *pass = NULL;
+
+  pass = getpwnam(user);
+  if (!pass)
+    {
+      strlower(user);
+      pass = getpwnam(user);
+    }
+
+  if (!pass) 
+    {
+      Debug(0,"Couldn't find user %s\n",user);
+      return(False);
+    }
+
+#ifdef PWDAUTH
+  pwok = (pwdauth(user,password) == 0);
+  if (!pwok)
+    {
+      strlower(password);
+      pwok = (pwdauth(user,password) == 0);
+    }	      
+  return(pwok);
+#endif
+
+#ifdef SHADOW_PWD
+  spass = getspnam(user);
+  if (spass && spass->sp_pwdp)
+    pass->pw_passwd = spass->sp_pwdp;
+#endif
+  strncpy(salt,pass->pw_passwd,2);
+  salt[2] = 0;
+  pwok = (strcmp(crypt(password,salt),pass->pw_passwd) == 0);
+  if (!pwok)
+    {
+      strlower(password);
+      pwok = (strcmp(crypt(password,salt),pass->pw_passwd) == 0);
+    }	      
+
+  return(pwok);
+}
+
+
+
+#ifdef HPUX
+/****************************************************************************
+this is a version of setbuffer() for those machines that only have setvbuf
+****************************************************************************/
+void setbuffer(FILE *f,char *buf,int bufsize)
+{
+  setvbuf(f,buf,_IOFBF,bufsize);
+}
+#endif
